/** @file ListAexcept.h 
 * Header file ListAexcept.h for the ADT list
 * Array-based implementation with exceptions
 * Carrano pages 156-8
 */
#ifndef _LISTA_H
#define _LISTA_H

//-----------------------
// C++ includes
//-----------------------
#include <string>
#include "ListException.h"
#include "ListIndexOutOfRangeException.h"

//----------------------
// Type definitions
//----------------------
typedef std::string ListItemType; 

//----------------------
// Global Constants
//----------------------
const int MAX_LIST = 10; /// @def maximum size of list  

/// List class (array-based). 
/**
 * Array-based implementation of a list with exceptions.
 * Modified from Carrano pages 156-8
 */
class List
{
public:
   /** Default constructor - sets list size to zero. */
   List(); 

   // destructor is supplied by compiler

   /**
    * Determines whether a list is empty.
    * @return True if list is empty, otherwise false.
    */ 
   bool isEmpty() const;

   /** Determines the length of a list.
    * @return The number of items currently in the list
    */
   int getLength() const;

   /**
    * Inserts item into list at specified index
    * @pre 1 <= index <= getLength() + 1.
    * @post If insertion is successful, newItem is at position index
    *       in the list, and other items are renumbered accordingly,
    * @param index  The list position at which to insert newItem.
    * @param newItem  The ListItemType to insert into the list.
    * @exception ListIndexOutOfRangeException index < 1 or index > getLength()+1
    * @exception ListException full array, so insertion fails.
    */
   void insert(int index, ListItemType newItem)
        throw(ListIndexOutOfRangeException, ListException);

   /** Deletes an item from the list at a given position.
    * @pre 1 <= index <= getLength().
    * @post If 1 <= index <= getLength(), the item at position index
    *       in the list is deleted, other items are renumbered accordingly.
    * @param index the position of the item to remove
    * @exception ListIndexOutOfRangeException index < 1 or index > getLength()
    */
   void remove(int index)
        throw(ListIndexOutOfRangeException);

   /** Retrieves a list item by position.
    * @pre 1 <= index <= getLength().
    * @post If 1 <= index <= getLength(), dataItem contains the desired item.
    * @param index  The list position to retrieve from.
    * @param dataItem  The ListItemType retrieved from the list.
    * @exception ListIndexOutOfRangeException index < 1 or index > getLength()
    */
   void retrieve(int index, ListItemType& dataItem) const
        throw(ListIndexOutOfRangeException);

private:
   /** array of list items */
   ListItemType items[MAX_LIST]; 
 
   /** number of items in list */
   int size;                    

   /** Converts the position of an item in a list to the 
    *  correct index within its array representation.
    *  @pre 1 <= index <= getLength().
    *  @param index  The list position to translate.
    *  @return The array index corresponding to list position index.*/ 
   int translate(int index) const;

}; // end List class
// End of header file.
 
#endif
