/** @file ListDriver.cpp
 * List Driver - use to test List class methods and 
 *  list functions that are independent from the List class.
 *                                
 * @author  Jeff Braun             
 * @version 0.92
 * @date 9/12/12
 **********************************************************/

//-----------------------
// C++ includes
//-----------------------
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>  //needed for atoi()

//-----------------------
// Application includes
//-----------------------
#include "ListAexcept.h"

//-----------------------
// Using statements
//-----------------------
using namespace std;

//----------------------
// Functions Prototypes
//----------------------
void displayList(List& list);   //display the list's entire contents
int getInt();   		//returns valid positive integer
ListItemType getItem();   	//prompts user for item 
void insertItem(List& list);    //Note the list is passed by reference
void menu();   			//displays driver menu
void processOption(char&, List&);     //processes menu input option
void restoreList(List& list);   //prompts for filename that contains items
void saveList(List& list); 	//prompts for filename to store items into

/** List Driver - use to test List class methods and 
 * list functions that are independent from the List class.
 **********************************************************/
int main()
{
  List aList;                 	// list to test
  string restoreListFlg;	// holds response to restore list from a file
  string menuOption;		// holds menu option input from user

//Opening Message
  cout<< "List Driver - Tests the List class" << endl;

//Restore the list from a file
  cout<< "----------------------------------" << endl; 
  cout<< "Would you like to restore the list from a file? (Y/N) ";
   
//Read user input - use a string to accept all inputs
  getline(cin, restoreListFlg);
  if(restoreListFlg[0] == 'Y' || restoreListFlg[0] == 'y'){ 
    restoreList(aList);            // opens/reads file into aList
  }

//Menu driven part of main program
//Initial menu display and loop to process menu option
  menu();
  do {
    cout << "\nEnter menu selection (0 for menu): ";
    getline(cin, menuOption);    //Use a string in case user enters chars
    processOption(menuOption[0], aList);
  }while(menuOption[0] != '8');
   
  return 0;            //Exit program
}//main()


//---------------------------
// Local function definitions
//---------------------------

/**
 * Displays the entire content of the list
 * @param[in] list to display
 * @remarks If list is empty, prints empty list message.
 */
void displayList(List& list) 
{
  int index;
  ListItemType item;

  for(index=1; index<=list.getLength(); index++){
    list.retrieve(index, item);
    cout << index << " - " << item << endl;
  }
  std::cout<<std::endl;
}

/**
  * Reads input string, validates it is an integer and returns it to caller
  * @return nonnegative integer
  * @remarks If input is invalid, then reprompts user for a positive int
  */
int getInt()   	
{
  string tmpStr;        	//input buffer
  bool isNumber = true;        //used for data validation
  int i=0;

  do{
    getline(cin, tmpStr);
    i=0;
    isNumber = true;
    while(i< (int)tmpStr.length() && isNumber){  //Validate integer input
      if(isalpha(tmpStr[i])){
        cout<<"Please enter a positive integer> ";
        isNumber = false;
      }
      i++;
    }
  } while(isNumber == false);

  return atoi(tmpStr.c_str());                //converts ascii to integer
}//getInt()

/**
  * Prompts for a list item and returns it to caller
  * @return list item
  * @remarks Accepts strings with spaces
  */
ListItemType getItem()
{
  ListItemType item;
  cout << "Enter the list item> ";
  cin >> ws;					//remove leading whitespace
  getline(cin, item);
  return item;
}

/**
  * Prompts users for index and item to insert, 
  * then inserts the items into list
  * @param[in,out] list with new item added
  * @post list with new item at specified index, 
  *  all items in list after index are shifted one. 
  * @remarks Prints error message if list is full.
  *  Prints error message if index not in valid range.
  */
void insertItem(List& list)   
{
  int index;
  ListItemType item;

  // I/O for user
  cout << "Insertion:" << endl;
  cout << "Enter the list index> ";
  index = getInt();
  item = getItem();

  //Insert new item into list
  try{
    list.insert(index, item);
  }
  catch(ListException e)
  {
    cout << e.what()<<endl;
    cout << "Please remove an item before inserting"<<endl;
  }

  //NOTE - You need to catch the second exception and handle it here

}//insertItem()

/**
  * Displays menu options
  */
void menu() 
{
   cout << "1) Print the length of the list" << endl;
   cout << "2) Insert an item into the list" << endl;
   cout << "3) Delete an item from the list" << endl;
   cout << "4) Retrieve an item (by index)" << endl;
   cout << "5) Search for an item (by value)" << endl;
   cout << "6) Reverse the list" << endl;
   cout << "7) Display the list" << endl;
   cout << "8) Quit the program" << endl;
}//menu()

/**
  * Process input menu option character from user.
  * @param[in] menuChar menu option character
  * @param[in,out] aList the list to be modified
  */
void processOption(char &menuChar, List &aList)
{
   string saveListFlg;

   switch (menuChar)
   {
    case '0':               // Display menu options
       menu();
       break;
    case '1':               // Output length of the list
       cout << "\nList Length is: " << aList.getLength() << endl;
       break;
    case '2':               // Prompt user for index
       insertItem(aList);   // and item to insert into list.
       break;

//NOTE that options 3 - 6 are left for the student to complete

    case '7':               // Display entire contents of list
       displayList(aList);
       break;
    case '8':               // Exit the program
    case 'Q':               // handles multiple quit inputs
    case 'q':
       cout<< "Would you like to save the list to a file? (Y/N) ";
       cin >> saveListFlg;
       if(saveListFlg[0] == 'Y' || saveListFlg[0] == 'y')
         saveList(aList);         // save list to file
       cout<< "Goodbye!"<< endl;  // Exit message
       menuChar = '8';
       break;
    default:                      // Invalid menu option entered
       cout << "\nError!  Invalid option.  Please try again.\n\n";
       menu();
    }
}//processOption()

/**
  * Restore the list from a user specified file
  * @param[inout] list restored from input file
  * @post The list contains all items in the file if size < max list length.
  * @exception ListException is not caught here, client should catch it.
  * @remarks Prompt the user for the filename to retore the list from.
  * If cannot open file, then reprompt user for filename.
  * Read items from file until end of file (eof) reached
  *  or until the list is full.
  * Print the number of items read from the file.
  */
void restoreList(List& list)
{
   ifstream fin;
   string filename;
   ListItemType item;
   int index=0;

   //Data validation to get filename and open it
   do{
     cout<<"Enter the filename that contains the list: ";
     getline(cin, filename);
     fin.clear();
     fin.open(filename.c_str());
     if(fin == 0){
       cout<<"ERROR  - could not open file " << filename<<endl;
     }
   } while (!fin);

   //read file until eof reached
   while(getline(fin,item)){
     index++;			//number of items successfully read
     list.insert(index, item);
   }
   cout<< endl << index << " items read into the list\n\n";
   fin.close();
}//restoreList()

/**
  * Prompts the user for the filename to store the list in.
  * @param[in] list to be saved to a user specified file
  */
void saveList(List& list)
{
   //NOTE this is a function stub. You need to implement it. 
   cout << "List will be saved when implemented" << endl;
}//saveList()

